<?php
/*
** Copyright (C) 2001-2026 Zabbix SIA
**
** This program is free software: you can redistribute it and/or modify it under the terms of
** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
**
** This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
** without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
** See the GNU Affero General Public License for more details.
**
** You should have received a copy of the GNU Affero General Public License along with this program.
** If not, see <https://www.gnu.org/licenses/>.
**/


/**
 * @var CView $this
 */
?>

<script type="text/javascript">
	const view = new class {
		init() {
			document.getElementById('userprofile-form').addEventListener('submit', (e) => {
				document.querySelectorAll('#autologout, #refresh, #url').forEach((elem) => {
					elem.value = elem.value.trim();
				});

				if (!this.#confirmSubmit()) {
					e.preventDefault();
				}
			});

			this.#changeVisibilityAutoLoginLogout();
			this.#autologoutHandler();
		}

		#confirmSubmit() {
			const elem_current = document.getElementById('current_password');
			const elem_password1 = document.getElementById('password1');
			const elem_password2 = document.getElementById('password2');

			if (elem_current && elem_password1 && elem_password2) {
				const current_password = elem_current.value;
				const password1 = elem_password1.value;
				const password2 = elem_password2.value;

				if (password1 !== '' && password2 !== '' && current_password !== '') {
					const warning_msg = <?= json_encode(
						_('In case of successful password change user will be logged out of all active sessions. Continue?')
					) ?>;

					return confirm(warning_msg);
				}
			}

			return true;
		}

		#changeVisibilityAutoLoginLogout() {
			const autologin_cbx = document.querySelector('#autologin');
			const autologout_cbx = document.querySelector('#autologout_visible');

			autologin_cbx.addEventListener('click', (e) => {
				if (e.target.checked) {
					autologout_cbx.checked = false;
				}
				this.#autologoutHandler();
			});

			autologout_cbx.addEventListener('click', (e) => {
				if (e.target.checked) {
					autologin_cbx.checked = false;
				}
				this.#autologoutHandler();
			});
		}

		#autologoutHandler() {
			const autologout_visible = document.querySelector('#autologout_visible');
			const disabled = !autologout_visible.checked;
			const autologout = document.querySelector('#autologout');
			const hidden = autologout.parentElement.
				querySelector(`input[type=hidden][name=${autologout.getAttribute('name')}]`);

			if (disabled) {
				autologout.setAttribute('disabled', '')
			}
			else {
				autologout.removeAttribute('disabled')
			}

			if (!hidden) {
				const hidden_input = document.createElement('input');

				hidden_input.type = 'hidden';
				hidden_input.name = autologout.getAttribute('name');
				hidden_input.value = '0';

				autologout.parentElement.insertBefore(hidden_input, autologout);
			}
			else if (!disabled) {
				hidden.remove();
			}
		}
	}
</script>
